classdef line_plot < fb_abstract.basic
% super class for linear and buffered plots

    % these properties can be changed by each individual plugin
    properties (SetAccess = public, GetAccess = public)
        % what color is used to plot the plugin
        plot_color = 'r';
        % line width used to plot the plugin
        LineWidth = 1;
    end

    % do not change!
    properties (Constant = true)
        % should the plot be rescaled automatically after the plugin is
        % initialized?
        PLOT_AUTOSCALE_INITIAL = 1;
    end
    

    %% public methods - none. We do not provide a constructor, because this
    % is an abstract class.
    methods


    end

    %% hidden methods
    methods (Access = protected, Hidden = true)

        function plugin_setup_plot(obj, Tag)

            %% use dummy data
            data = obj.TEST_DATA;

            %% setup the actual plot

            plot_handle = stairs((1:size(data, 2))/obj.Scanrate, data, ...
                'Parent', obj.ParentAxis, ...
                'Tag', Tag, ...
                'Color', obj.plot_color, ...
                'LineWidth', obj.LineWidth);

            % save handle
            obj.PluginHandle = plot_handle;

            %% set the properties of the parent axis accordingly.

            % do not use 'Visible', 'off', ..., because this will remove
            % the "axis color" = background color of the plot.

            set(obj.ParentAxis, 'Box', 'off', ...
                'XLim', [0 1], ...
                'XTick', []);


        end

        function class_hide_handles(obj, src, evnt) %#ok<INUSD>
            set(obj.PluginHandle, 'Visible', 'off');
            handle_children = get(obj.PluginHandleGroup, 'Children');
            set(handle_children, 'Visible', 'off')
        end

        function class_show_handles(obj, src, evnt) %#ok<INUSD>
            set(obj.PluginHandle, 'Visible', 'on');
            handle_children = get(obj.PluginHandleGroup, 'Children');
            set(handle_children, 'Visible', 'on')
        end

        function plugin_createContextMenuPlugin(obj) %#ok<MANU>
        end

        function class_plot_routine(obj, data)

            %% keep ROI only
            roi = obj.ROI;
            % make sure we keep multiple channels (if they exist)
            data = data(:, roi(1):roi(2));
            
            %% set the correct x-axis limits for line plots.
            obj.set_xlims(size(data, 2));
            
            %% scale and transform data
            data = obj.scale_transform_data(data);

            %% now do the actual plotting

            obj.custom_plot(data);
            obj.update_y_ticks();
            
        end
    end

    %%


end
%% EOF