function tag_GUI_setup_menu(obj, contextmenu)

    % my vertical line_object
    obj.TAG_GUI_OBJECT = plots.line_vertical_group(obj.ParentAxis, obj.FBMainWindow, obj.ROI, obj.Scanrate);

    %% tagging MAIN menu
    obj.TAG_GUI_contextmenu = uimenu(contextmenu, ...
        'Label', '&Tagging', ...
        'Separator', 'On', ...
        'Callback', '');

    %% menu ENTRIES
    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'Run ''tag_data()''', ...
        'Callback', @(src, evnt)context_tag_data(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'Set ''TAG_THRESHOLD'', run ''tag_data()'' && (re-)plot ''TAG_INDICES''', ...
        'Callback', @(src, evnt)set_threshold_tag_plot(obj, src, evnt), ...
        'Tag', 'tag_GUI_setup_menu_set_threshold_tag_plot');

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', '&Run ''tag_data()'' && print ''TAG_INDICES'' (to command line)', ...
        'Callback', @(src, evnt)tag_data_print_indices(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'change ''TAG_THRESHOLD''', ...
        'Separator', 'On', ...
        'Callback', @(src, evnt)change_threshold(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'set ''TAG_THRESHOLD'' using the Horizontal Line position', ...
        'Callback', @(src, evnt)change_threshold_from_horizontal_line(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'print ''TAG_INDICES'' (to command line)', ...
        'Separator', 'On', ...
        'Callback', @(src, evnt)print_indices(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'print ''TAG_THRESHOLD'' (to command line)', ...
        'Callback', @(src, evnt)print_threshold(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'clear ''TAG_INDICES''', ...
        'Callback', @(src, evnt)clear_indices(obj, src, evnt));

    % PLOT THRESHOLD CROSSINGS - crude implementation only
    checked = 'off';
    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'plot ''TAG_INDICES''', ...
        'Checked', checked, ...
        'Separator', 'On', ...
        'Tag', 'TAG_PLOT_MENU_ITEM', ...
        'Callback', @(src, evnt)run_plot_indices(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 're-plot ''TAG_INDICES''', ...
        'Callback', @(src, evnt)re_plot_indices(obj, src, evnt));

    uimenu(obj.TAG_GUI_contextmenu, ...
        'Label', 'hide ''TAG_INDICES''', ...
        'Callback', @(src, evnt)hide_vertical_lines(obj, src, evnt));


%% TAGGING CALLBACKS


    function hide_vertical_lines(obj, src, evnt) %#ok<INUSD>
        obj.TAG_GUI_OBJECT.hide_handles();
        set(src, 'Label', 'show ''TAG_INDICES''', ...
            'Callback', @(src, evnt)show_vertical_lines(obj, src, evnt));
    end

    function show_vertical_lines(obj, src, evnt) %#ok<INUSD>
        obj.TAG_GUI_OBJECT.show_handles();
        set(src, 'Label', 'hide ''TAG_INDICES''', ...
            'Callback', @(src, evnt)hide_vertical_lines(obj, src, evnt));
    end

    function context_tag_data(obj, src, evnt) %#ok<INUSD>
        
        % data type in FB is always DOUBLE
        obj.TAG_DATA_TYPE = 1;
        obj.TAG_EXTRACT_BUFFERS = 0;
        obj.tag_data(obj.data);
        
    end

    function tag_data_print_indices(obj, src, evnt)
        context_tag_data(obj, src, evnt);
        print_indices(obj, src, evnt);
    end

    function set_threshold_tag_plot(obj, src, evnt)
        change_threshold_from_horizontal_line(obj, src, evnt);
        % manually check for a valid threshold here, so we can skip all
        % the remaining function calls, which will lead to confusing error
        % messages that are only meant for scripting mode.
        if isnan(obj.TAG_THRESHOLD)
            return;
        end
        context_tag_data(obj, src, evnt);
        re_plot_indices(obj, src, evnt);
    end

    function print_indices(obj, src, evnt) %#ok<INUSD>
        
        if obj.TAG_FOUND == 0
            disp('No ON or OFF crossings.');
            return;
        end
        
        if isempty(obj.TAG_INDICES) || isempty(obj.TAG_INDICES_OFF)
            disp('ON crossings:');
            disp(obj.TAG_INDICES);
            disp('OFF crossings:');
            disp(obj.TAG_INDICES_OFF);
        else
            disp('ON / OFF crossings:');
            disp([obj.TAG_INDICES' obj.TAG_INDICES_OFF']);
        end
    end

    function print_threshold(obj, src, evnt) %#ok<INUSD>
        disp(['''TAG_THRESHOLD'': ' num2str(obj.TAG_THRESHOLD)]);
    end

    function clear_indices(obj, src, evnt) %#ok<INUSD>
        obj.tag_clear();
    end

    function change_threshold(obj, src, evnt) %#ok<INUSD>
        fb_GUI_ChangeValue(obj, 'TAG_THRESHOLD', [0 inf], 'Tagging threshold (0-inf)');
    end

    function change_threshold_from_horizontal_line(obj, src, evnt) %#ok<INUSD>
        % main horizontal line.
        line_handle = findobj(obj.ParentAxis, 'Tag', 'HorizontalLineOne');
        % additional horizontal line.
        line_handle_two = findobj(obj.ParentAxis, 'Tag', 'HorizontalLineTwo');

        if isempty(line_handle)
            disp('Could not find the main horizontal line (green color by default), which is used for tagging.');
            if ~isempty(line_handle_two)
                disp('It seems like you have activated the second horizontal line - turn it off and enable the main horizontal line instead.');
            end
            if ~isnan(obj.TAG_THRESHOLD)
                disp(['Keeping old ''TAG_THRESHOLD'' value ' num2str(obj.TAG_THRESHOLD) ]);
            end
            return;
        end
        
        curr_pos = get(line_handle, 'YData');
        new_threshold = curr_pos(1);
        obj.TAG_THRESHOLD = new_threshold;
        disp(['''TAG_THRESHOLD'' is set to ' num2str(new_threshold)]);
        
    end

%% PLOT THRESHOLD CROSSINGS - crude implementation only
% does not automatically replot if you zoom
    function run_plot_indices(obj, src, evnt) %#ok<INUSD>
        
        if obj.TAG_FOUND == 0
            disp('Please run ''tag_data()'' first.');
            return;
        end
        
        TAG_PLOT_MENU_ITEM = findobj('Tag', 'TAG_PLOT_MENU_ITEM');
        new_value = invert_checked(TAG_PLOT_MENU_ITEM);
        if strcmp(new_value, 'off')
            obj.TAG_GUI_ACTIVATED = 0;
            delete_plotted_indices(obj)
            return;
        end
        
        plot_indices(obj);
        
        
    end

%% this is a quick & dirty implementation. it would be better to use
% something like fb_plugin_show_timepoints_from_flatclust()

    function plot_indices(obj)
        
        obj.TAG_GUI_ACTIVATED = 1;
        
        if obj.TAG_FOUND == 0
            disp('Please run ''tag_data()'' first.');
            return;
        end
        
        % update the time stamps
        obj.TAG_GUI_OBJECT.update_indices(obj.TAG_INDICES);
        % update the plotting
        obj.TAG_GUI_OBJECT.update(obj.ROI, obj.Scanrate);
        % don't use setup_line here, because we need the ROI and Scanrate!
        % obj.TAG_GUI_OBJECT.setup_lines()
        
    end

    function delete_plotted_indices(obj)
        obj.TAG_GUI_OBJECT.delete_plotted_indices()
    end

    function re_plot_indices(obj, src, evnt) %#ok<INUSD>
        TAG_PLOT_MENU_ITEM = findobj('Tag', 'TAG_PLOT_MENU_ITEM');
        set(TAG_PLOT_MENU_ITEM, 'Checked', 'On');
        delete_plotted_indices(obj);
        plot_indices(obj);
    end

end
%% EOF
